--
-- **************************************************************************
-- **************************************************************************
-- **                                                                      **
-- **                      "Ukady CPLD w praktyce"                        **
-- **                                                                      **
-- **    Project name: BinCntDE                                            **
-- **    Module: bincntde (root)                                           **
-- **    Target device: CoolRunner II (XC2C256/TQ144) [DIGILENT]           **
-- **    Tool versions: ISE WebPACK 9.2i                                   **
-- **    Date: BIAYSTOK, stycze 2009                                     **
-- **                                                                      **
-- **                     Napisa : Andrzej Pawluczuk                      **
-- **                                                                      **
-- **************************************************************************
-- **************************************************************************
--
-----------------------------------------------------------------------------
----  Prezentacja dziaania przerzutnikw reagujcych jednoczenie
----  na narastajce i opadajce zbocze zegarowe.
----  Realizacja moliwa jest tylko w ukadach CoolRunner II.
-----------------------------------------------------------------------------
library IEEE ;
use IEEE.STD_LOGIC_1164.ALL ;
use IEEE.STD_LOGIC_ARITH.ALL ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;

entity BinCntDE is port ( Clk : in std_logic ;
                          Reset : in std_logic ;
                          ClkLED : out std_logic ;
                          Seg_A : out std_logic ;
                          Seg_B : out std_logic ;
                          Seg_C : out std_logic ;
                          Seg_D : out std_logic ;
                          Seg_E : out std_logic ;
                          Seg_F : out std_logic ;
                          Seg_G : out std_logic ;
                          Seg_K : out std_logic ;
                          DSel0 : out std_logic ;
                          DSel1 : out std_logic ;
                          DSel2 : out std_logic ;
                          DSel3 : out std_logic ) ;
end BinCntDE ;

architecture Behavioral of BinCntDE is

  component Mux4DigDisp port ( DClk : in std_logic ;
                               Reset : in std_logic ;
                               Neg : in std_logic := '0' ;
                               WA : in std_logic := '0' ;
                               Tetr0 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr1 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr2 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr3 : in std_logic_vector ( 3 downto 0 ) ;
                               Dot   : in std_logic_vector ( 3 downto 0 ) ;
                               Seg_A : out std_logic ;
                               Seg_B : out std_logic ;
                               Seg_C : out std_logic ;
                               Seg_D : out std_logic ;
                               Seg_E : out std_logic ;
                               Seg_F : out std_logic ;
                               Seg_G : out std_logic ;
                               Seg_K : out std_logic ;
                               DSel0 : out std_logic ;
                               DSel1 : out std_logic ;
                               DSel2 : out std_logic ;
                               DSel3 : out std_logic ) ;
  end component ;

  signal DClk : std_logic ;
  signal CClk : std_logic ;
  signal Cnt : std_logic_vector ( 7 downto 0 ) ;
  signal CntDE : std_logic_vector ( 7 downto 0 ) ;
  signal FDiv : std_logic_vector ( 19 downto 0 ) ;

begin
  FDivInstance : process ( Clk , Reset )
  begin
    if Reset = '0' then
      FDiv <= ( others => '0' ) ;
    else
      if Clk'event and Clk = '0' then
        FDiv <= FDiv + 1 ;
      end if ;
    end if ;
  end process ;

  DClk <= FDiv ( 6 ) ;

  CClk <= FDiv ( FDiv'left ) ;

  ClkLED <= not CClk ;

  CntInstance : process ( CClk , Reset )
  begin
    if Reset = '0' then
      Cnt <= ( others => '0' ) ;
    else
      if CClk'event and CClk = '0' then
        Cnt <= Cnt + 1 ;
      end if ;
    end if ;
  end process ;

  CntDEInstance : process ( CClk , Reset )
  begin
    if Reset = '0' then
      CntDE <= ( others => '0' ) ;
    else
      if CClk'event then
        CntDE <= CntDE + 1 ;
      end if ;
    end if ;
  end process ;

  MuxDisplInstance : Mux4DigDisp port map ( DClk => DClk ,
                                            Reset => Reset ,
                                            Tetr0 => CntDE ( 3 downto 0 ) ,
                                            Tetr1 => CntDE ( 7 downto 4 ) ,
                                            Tetr2 => Cnt ( 3 downto 0 ) ,
                                            Tetr3 => Cnt ( 7 downto 4 ) ,
                                            Dot => "1011" ,
                                            Seg_A => Seg_A ,
                                            Seg_B => Seg_B ,
                                            Seg_C => Seg_C ,
                                            Seg_D => Seg_D ,
                                            Seg_E => Seg_E ,
                                            Seg_F => Seg_F ,
                                            Seg_G => Seg_G ,
                                            Seg_K => Seg_K ,
                                            DSel0 => DSel0 ,
                                            DSel1 => DSel1 ,
                                            DSel2 => DSel2 ,
                                            DSel3 => DSel3 ) ;

end Behavioral ;
