--
-- **************************************************************************
-- **************************************************************************
-- **                                                                      **
-- **                      "Ukady CPLD w praktyce"                        **
-- **                                                                      **
-- **    Project name: Freq                                                **
-- **    Module: Freq (root)                                               **
-- **    Target device: CoolRunner II (XC2C256/TQ144) [DIGILENT]           **
-- **    Tool versions: ISE WebPACK 9.2i                                   **
-- **    Date: BIAYSTOK, stycze 2009                                     **
-- **                                                                      **
-- **                     Napisa : Andrzej Pawluczuk                      **
-- **                                                                      **
-- **************************************************************************
-- **************************************************************************
--
-----------------------------------------------------------------------------
library IEEE ;
use IEEE.STD_LOGIC_1164.ALL ;
use IEEE.STD_LOGIC_ARITH.ALL ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;

entity Freq is port ( Clk : in std_logic ;
                      Reset : in std_logic ;
                      FInp : in std_logic ;
                      GoKey : in std_logic ;
                      RangeKey : in std_logic ;
                      RunLed : out std_logic ;
                      Seg_A : out std_logic ;
                      Seg_B : out std_logic ;
                      Seg_C : out std_logic ;
                      Seg_D : out std_logic ;
                      Seg_E : out std_logic ;
                      Seg_F : out std_logic ;
                      Seg_G : out std_logic ;
                      Seg_K : out std_logic ;
                      DSel0 : out std_logic ;
                      DSel1 : out std_logic ;
                      DSel2 : out std_logic ;
                      DSel3 : out std_logic ) ;
end Freq ;

architecture Behavioral of Freq is

  constant FDivCntSizeParam : integer := 8 ;
  constant MuxDisplNegParam : std_logic := '0' ;
  constant MuxDisplWAParam : std_logic := '0' ;

  component FDivider generic ( constant CntSize : integer := 8 ) ;
                     port ( Clk : in std_logic ;
                            Reset : in std_logic ;
                            FDiv : out std_logic ) ;
  end component ;

  component BCD6DigCnt port ( Clk : in std_logic ;
                              Reset : in std_logic ;
                              Enable : in std_logic ;
                              Cnt1 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt10 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt100 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt1000 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt10000 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt100000 : out std_logic_vector ( 3 downto 0 ) ) ;
  end component ;

  component Mux4DigDispBl port ( DClk : in std_logic ;
                                 Reset : in std_logic ;
                                 Neg : in std_logic := '0' ;
                                 WA : in std_logic := '0' ;
                                 Tetr0 : in std_logic_vector ( 3 downto 0 ) ;
                                 Tetr1 : in std_logic_vector ( 3 downto 0 ) ;
                                 Tetr2 : in std_logic_vector ( 3 downto 0 ) ;
                                 Tetr3 : in std_logic_vector ( 3 downto 0 ) ;
                                 Dot   : in std_logic_vector ( 3 downto 0 ) ;
                                 Seg_A : out std_logic ;
                                 Seg_B : out std_logic ;
                                 Seg_C : out std_logic ;
                                 Seg_D : out std_logic ;
                                 Seg_E : out std_logic ;
                                 Seg_F : out std_logic ;
                                 Seg_G : out std_logic ;
                                 Seg_K : out std_logic ;
                                 DSel0 : out std_logic ;
                                 DSel1 : out std_logic ;
                                 DSel2 : out std_logic ;
                                 DSel3 : out std_logic ) ;
  end component ;

  component PushButton is port ( KClk : in std_logic ;
                                 Reset : in std_logic ;
                                 KeyIn : in std_logic ;
                                 Status : out std_logic ) ;
  end component ;

  type AutomatStateType is ( IdleState ,
                             RangeOptionState ,
                             GoOptionState ,
                             RunState ,
                             StartRunState ,
                             StopRunState ,
                             EndOptionState ) ;

  signal MachineState : AutomatStateType ;
  signal Cnt1 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt10 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt100 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt1000 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt10000 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt100000 : std_logic_vector ( 3 downto 0 ) ;
  signal R1 : std_logic_vector ( 3 downto 0 ) ;
  signal R10 : std_logic_vector ( 3 downto 0 ) ;
  signal R100 : std_logic_vector ( 3 downto 0 ) ;
  signal R1000 : std_logic_vector ( 3 downto 0 ) ;
  signal R10000 : std_logic_vector ( 3 downto 0 ) ;
  signal R100000 : std_logic_vector ( 3 downto 0 ) ;
  signal Tetr0 : std_logic_vector ( 3 downto 0 ) ;
  signal Tetr1 : std_logic_vector ( 3 downto 0 ) ;
  signal Tetr2 : std_logic_vector ( 3 downto 0 ) ;
  signal Tetr3 : std_logic_vector ( 3 downto 0 ) ;
  signal DClk : std_logic ;
  signal KClk : std_logic ;
  signal FClk : std_logic ;
  signal FDiv : std_logic_vector ( 3 downto 0 ) ;
  signal GoKeyStatus : std_logic ;
  signal RangeKeyStatus : std_logic ;
  signal Dot : std_logic_vector ( 3 downto 0 ) ;
  signal F1HzCnt : std_logic_vector ( 19 downto 0 ) ;
  signal F1HzEnable : std_logic ;
  signal F1Hz : std_logic ;
  signal T1S : std_logic ;
  signal HzRange : std_logic ;
  signal CntEnable : std_logic ;
  signal ClearCnt : std_logic ;
  signal Clr : std_logic ;

begin
  FDivInstance : FDivider generic map ( CntSize => FDivCntSizeParam )
                          port map ( Clk => Clk ,
                                     Reset => Reset ,
                                     FDiv => FClk ) ;

  ClkGenInstance : process ( FClk , Reset )
  begin
    if Reset = '0' then
      FDiv <= ( others => '0' ) ;
    else
      if FClk'event and FClk = '0' then
        FDiv <= FDiv + 1 ;
      end if ;
    end if ;
  end process ;

  DClk <= FDiv ( 3 ) ;
  KClk <= FDiv ( 2 ) ;

  F1HzGenInstance : process ( Clk , F1HzEnable , ClearCnt )
  begin
    if ClearCnt = '0' then
      F1HzCnt <= ( others => '1' ) ;
    else
      if Clk'event and Clk = '0' then
        if F1HzEnable = '1' then
          if F1HzCnt = "11110100001000111111" then
            F1HzCnt <= ( others => '0' ) ;
          else
            F1HzCnt <= F1HzCnt + 1 ;
          end if ;
        end if ;
      end if ;
    end if ;
  end process ;

  F1Hz <= F1HzCnt ( F1HzCnt'left ) ;

  T1SGenInstance : process ( F1Hz , F1HzEnable , ClearCnt )
  begin
    if ClearCnt = '0' then
      T1S <= '1' ;
    else
      if F1Hz'event and F1Hz = '0' then
        if F1HzEnable = '1' then
          T1S <= not T1S ;
        end if ;
      end if ;
    end if ;
  end process ;
  CntEnable <= not T1S ;

  ClearCnt <= '0' when ( Reset = '0' or Clr = '0' ) else '1' ;

  GoKInstance : PushButton port map ( KClk => KClk ,
                                      Reset => Reset ,
                                      KeyIn => GoKey ,
                                      Status => GoKeyStatus ) ;

  RangeKInstance : PushButton port map ( KClk => KClk ,
                                         Reset => Reset ,
                                         KeyIn => RangeKey ,
                                         Status => RangeKeyStatus ) ;

  BCDCntInstance : BCD6DigCnt port map ( Clk => FInp ,
                                         Reset => ClearCnt ,
                                         Enable => CntEnable ,
                                         Cnt1 => Cnt1 ,
                                         Cnt10 => Cnt10 ,
                                         Cnt100 => Cnt100 ,
                                         Cnt1000 => Cnt1000 ,
                                         Cnt10000 => Cnt10000 ,
                                         Cnt100000 => Cnt100000 ) ;

  with HzRange select
    Dot <= "1110" when '1' ,
           "1101" when others ;
  with HzRange select
    Tetr0 <= R1 when '1' ,
             R100 when others ;
  with HzRange select
    Tetr1 <= R10 when '1' ,
             R1000 when others ;
  with HzRange select
    Tetr2 <= R100 when '1' ,
             R10000 when others ;
  with HzRange select
    Tetr3 <= R1000 when '1' ,
             R100000 when others ;

  MuxDisplInstance : Mux4DigDispBl port map ( DClk => DClk ,
                                              Reset => Reset ,
                                              Neg => MuxDisplNegParam ,
                                              WA => MuxDisplWAParam ,
                                              Tetr0 => Tetr0 ,
                                              Tetr1 => Tetr1 ,
                                              Tetr2 => Tetr2 ,
                                              Tetr3 => Tetr3 ,
                                              Dot => Dot ,
                                              Seg_A => Seg_A ,
                                              Seg_B => Seg_B ,
                                              Seg_C => Seg_C ,
                                              Seg_D => Seg_D ,
                                              Seg_E => Seg_E ,
                                              Seg_F => Seg_F ,
                                              Seg_G => Seg_G ,
                                              Seg_K => Seg_K ,
                                              DSel0 => DSel0 ,
                                              DSel1 => DSel1 ,
                                              DSel2 => DSel2 ,
                                              DSel3 => DSel3 ) ;

  AutomatInstance : process ( Clk , Reset )
  begin
    if Reset = '0' then
      HzRange <= '1' ;
      MachineState <= IdleState ;
      R1 <= ( others => '0' ) ;
      R10 <= ( others => '0' ) ;
      R100 <= ( others => '0' ) ;
      R1000 <= ( others => '0' ) ;
      R10000 <= ( others => '0' ) ;
      R100000 <= ( others => '0' ) ;
    else
      if Clk'event and Clk = '0' then
        case MachineState is
          when IdleState =>
            F1HzEnable <= '0' ;
            Clr <= '1' ;
            if GoKeyStatus = '1' then
              MachineState <= GoOptionState ;
            elsif RangeKeyStatus = '1' then
              MachineState <= RangeOptionState ;
            else
              MachineState <= IdleState ;
            end if ;
          when RangeOptionState =>
            Clr <= '1' ;
            F1HzEnable <= '0' ;
            HzRange <= not HzRange ;
            MachineState <= EndOptionState ;
          when GoOptionState =>
            Clr <= '0' ;
            F1HzEnable <= '1' ;
            MachineState <= StartRunState ;
          when StartRunState =>
            Clr <= '1' ;
            F1HzEnable <= '1' ;
            if T1S = '0' then
              MachineState <= RunState ;
            else
              MachineState <= StartRunState ;
            end if ;
          when RunState =>
            Clr <= '1' ;
            F1HzEnable <= '1' ;
            if T1S = '0' then
              MachineState <= RunState ;
            else
              MachineState <= StopRunState ;
            end if ;
          when StopRunState =>
            Clr <= '1' ;
            F1HzEnable <= '0' ;
            R1 <= Cnt1 ;
            R10 <= Cnt10 ;
            R100 <= Cnt100 ;
            R1000 <= Cnt1000 ;
            R10000 <= Cnt10000 ;
            R100000 <= Cnt100000 ;
            MachineState <= EndOptionState ;
          when EndOptionState =>
            Clr <= '1' ;
            F1HzEnable <= '0' ;
            if GoKeyStatus = '0' and RangeKeyStatus = '0' then
              MachineState <= IdleState ;
            else
              MachineState <= EndOptionState ;
            end if ;
        end case ;
      end if ;
    end if ;
  end process ;

  RunLed <= T1S xor MuxDisplWAParam ;
end Behavioral ;
