--
-- **************************************************************************
-- **************************************************************************
-- **                                                                      **
-- **                      "Ukady CPLD w praktyce"                        **
-- **                                                                      **
-- **    Project name: Generator                                           **
-- **    Module: Generator (root)                                          **
-- **    Target device: CoolRunner II (XC2C256/TQ144) [DIGILENT]           **
-- **    Tool versions: ISE WebPACK 9.2i                                   **
-- **    Date: BIAYSTOK, stycze 2009                                     **
-- **                                                                      **
-- **                     Napisa : Andrzej Pawluczuk                      **
-- **                                                                      **
-- **************************************************************************
-- **************************************************************************
--
-----------------------------------------------------------------------------
library IEEE ;
use IEEE.STD_LOGIC_1164.ALL ;
use IEEE.STD_LOGIC_ARITH.ALL ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;

entity Generator is port ( Clk : in std_logic ;
                           Reset : in std_logic ;
                           ModeKeyInp : in std_logic ;
                           DirKeyInp : in std_logic ;
                           SetKeyInp : in std_logic ;
                           OutSig : out std_logic ;
                           DirLed : out std_logic ;
                           Seg_A : out std_logic ;
                           Seg_B : out std_logic ;
                           Seg_C : out std_logic ;
                           Seg_D : out std_logic ;
                           Seg_E : out std_logic ;
                           Seg_F : out std_logic ;
                           Seg_G : out std_logic ;
                           Seg_K : out std_logic ;
                           DSel0 : out std_logic ;
                           DSel1 : out std_logic ;
                           DSel2 : out std_logic ;
                           DSel3 : out std_logic ) ;
end Generator ;

architecture Behavioral of Generator is

  constant FDivCntSizeParam : integer := 8 ;
  constant DisplayNegParam : std_logic := '0' ;
  constant DisplayWAParam : std_logic := '0' ;

  component FDivider generic ( constant CntSize : integer := 8 ) ;
                     port ( Clk : in std_logic ;
                            Reset : in std_logic ;
                            FDiv : out std_logic ) ;
  end component ;

  component PushButton port ( KClk : in std_logic ;
                              Reset : in std_logic ;
                              KeyIn : in std_logic ;
                              Status : out std_logic ) ;
  end component ;

  component ARepKSrv port ( Clk : in std_logic ;
                            TClk : in std_logic ;
                            Reset : in std_logic ;
                            KeyIn : in std_logic ;
                            Status : out std_logic ) ;
  end component ;

  component Mux4DigDisp port ( DClk : in std_logic ;
                               Reset : in std_logic ;
                               Neg : in std_logic := '0' ;
                               WA : in std_logic := '0' ;
                               Tetr0 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr1 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr2 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr3 : in std_logic_vector ( 3 downto 0 ) ;
                               Dot   : in std_logic_vector ( 3 downto 0 ) ;
                               Seg_A : out std_logic ;
                               Seg_B : out std_logic ;
                               Seg_C : out std_logic ;
                               Seg_D : out std_logic ;
                               Seg_E : out std_logic ;
                               Seg_F : out std_logic ;
                               Seg_G : out std_logic ;
                               Seg_K : out std_logic ;
                               DSel0 : out std_logic ;
                               DSel1 : out std_logic ;
                               DSel2 : out std_logic ;
                               DSel3 : out std_logic ) ;
  end component ;

  type AutomatStateType is ( IdleState ,
                             SetOptionState ,
                             DirOptionState ,
                             EndOptionState ) ;

  signal MachineState : AutomatStateType ;
  signal LCmpReg : std_logic_vector ( 7 downto 0 ) ;
  signal UCmpReg : std_logic_vector ( 7 downto 0 ) ;
  signal BinCnt : std_logic_vector ( 7 downto 0 ) ;
  signal FDiv : std_logic_vector ( 6 downto 0 ) ;
  signal DotControl : std_logic_vector ( 3 downto 0 ) ;
  signal Context : std_logic_vector ( 1 downto 0 ) ;
  signal DClk : std_logic ;
  signal KClk : std_logic ;
  signal FClk : std_logic ;
  signal TClk : std_logic ;
  signal PClk : std_logic ;
  signal LRegEnable : std_logic ;
  signal URegEnable : std_logic ;
  signal CntEnable : std_logic ;
  signal OutFF : std_logic ;
  signal DirFF : std_logic ;
  signal SetKeyStatus : std_logic ;
  signal ModeKeyStatus : std_logic ;
  signal DirKeyStatus : std_logic ;

begin
  FDivInstance : FDivider generic map ( CntSize => FDivCntSizeParam )
                          port map ( Clk => Clk ,
                                     Reset => Reset ,
                                     FDiv => FClk ) ;

  ClkGenInstance : process ( FClk , Reset )
  begin
    if Reset = '0' then
      FDiv <= ( others => '0' ) ;
    else
      if FClk'event and FClk = '0' then
        FDiv <= FDiv + 1 ;
      end if ;
    end if ;
  end process ;

  DClk <= FDiv ( 3 ) ;
  KClk <= FDiv ( 4 ) ;
  TClk <= FDiv ( 6 ) ;

  ModeKeyInstance : PushButton port map ( KClk => KClk ,
                                          Reset => Reset ,
                                          KeyIn => ModeKeyInp ,
                                          Status => ModeKeyStatus ) ;

  DirKeyInstance : PushButton port map ( KClk => KClk ,
                                         Reset => Reset ,
                                         KeyIn => DirKeyInp ,
                                         Status => DirKeyStatus ) ;

  SetupKeyInstance : ARepKSrv port map ( Clk => Clk ,
                                         TClk => TClk ,
                                         Reset => Reset ,
                                         KeyIn => SetKeyInp ,
                                         Status => SetKeyStatus ) ;

  LRegEnable <= '1' when Context = "01" else '0' ;
  URegEnable <= '1' when Context = "10" else '0' ;
  CntEnable <= '1' when Context = "00" else '0' ;

  LRegInstance : process ( SetKeyStatus , Reset , LRegEnable , DirFF )
  begin
    if Reset = '0' then
      LCmpReg <= ( '1' , others => '0' ) ;
    else
      if SetKeyStatus'event and SetKeyStatus = '0' then
        if LRegEnable = '1' then
          if DirFF = '0' then
            LCmpReg <= LCmpReg + 1 ;
          else
            LCmpReg <= LCmpReg - 1 ;
          end if ;
        end if ;
      end if ;
    end if ;
  end process ;

  URegInstance : process ( SetKeyStatus , Reset , URegEnable , DirFF )
  begin
    if Reset = '0' then
      UCmpReg <= ( '1' , others => '0' ) ;
    else
      if SetKeyStatus'event and SetKeyStatus = '0' then
        if URegEnable = '1' then
          if DirFF = '0' then
            UCmpReg <= UCmpReg + 1 ;
          else
            UCmpReg <= UCmpReg - 1 ;
          end if ;
        end if ;
      end if ;
    end if ;
  end process ;

  PClk <= FDiv ( 3 ) ;

  CntInstance : process ( PClk , CntEnable , Reset )
  begin
    if Reset = '0' then
      OutFF <= '0' ;
      BinCnt <= ( others => '0' ) ;
    else
      if CntEnable = '1' then
        if PClk'event and PClk = '0' then
          if OutFF = '0' then
            if BinCnt = LCmpReg then
              OutFF <= '1' ;
              BinCnt <= ( others => '0' ) ;
            else
              BinCnt <= BinCnt + 1 ;
            end if ;
          else
            if BinCnt = UCmpReg then
              OutFF <= '0' ;
              BinCnt <= ( others => '0' ) ;
            else
              BinCnt <= BinCnt + 1 ;
            end if ;
          end if ;
        end if ;
      end if ;
    end if ;
  end process ;

  with Context select
    DotControl <= "0011" when "10" ,
                  "1100" when "01" ,
                  "1111" when others ;

  MuxDisplInstance : Mux4DigDisp port map ( DClk => DClk ,
                                            Reset => Reset ,
                                            Neg => DisplayNegParam ,
                                            WA => DisplayWAParam ,
                                            Tetr0 => LCmpReg ( 3 downto 0 ) ,
                                            Tetr1 => LCmpReg ( 7 downto 4 ) ,
                                            Tetr2 => UCmpReg ( 3 downto 0 ) ,
                                            Tetr3 => UCmpReg ( 7 downto 4 ) ,
                                            Dot => DotControl ,
                                            Seg_A => Seg_A ,
                                            Seg_B => Seg_B ,
                                            Seg_C => Seg_C ,
                                            Seg_D => Seg_D ,
                                            Seg_E => Seg_E ,
                                            Seg_F => Seg_F ,
                                            Seg_G => Seg_G ,
                                            Seg_K => Seg_K ,
                                            DSel0 => DSel0 ,
                                            DSel1 => DSel1 ,
                                            DSel2 => DSel2 ,
                                            DSel3 => DSel3 ) ;

  AutomatInstance : process ( Clk , Reset )
  begin
    if Reset = '0' then
      MachineState <= IdleState ;
      Context <= "00" ;
      DirFF <= '0' ;
    else
      if Clk'event and Clk = '0' then
        case MachineState is
          when IdleState =>
            if ModeKeyStatus = '1' then
              MachineState <= SetOptionState ;
            elsif DirKeyStatus = '1' then
              MachineState <= DirOptionState ;
            else
              MachineState <= IdleState ;
            end if ;
          when SetOptionState =>
            if Context = "00" then
              Context <= "01" ;
            elsif Context = "01" then
              Context <= "10" ;
            elsif Context = "10" then
              Context <= "00" ;
            end if ;
            MachineState <= EndOptionState ;
          when DirOptionState =>
            DirFF <= not DirFF ;
            MachineState <= EndOptionState ;
          when EndOptionState =>
            if ModeKeyStatus = '0' and DirKeyStatus = '0' then
              MachineState <= IdleState ;
            else
              MachineState <= EndOptionState ;
            end if ;
        end case ;
      end if ;
    end if ;
  end process ;

  OutSig <= OutFF ;
  DirLed <= not DirFF ;
end Behavioral ;
