--
-- **************************************************************************
-- **************************************************************************
-- **                                                                      **
-- **             "Ukady programowalne CoolRunner II w praktyce"          **
-- **                                                                      **
-- **    Project name: Stoper                                              **
-- **    Module: Stoper (root)                                             **
-- **    Target device: CoolRunner II (XC2C256/TQ144) [DIGILENT]           **
-- **    Tool versions: ISE WebPACK 9.2i                                   **
-- **    Date: BIAYSTOK, stycze 2009                                     **
-- **                                                                      **
-- **                     Napisa : Andrzej Pawluczuk                      **
-- **                                                                      **
-- **************************************************************************
-- **************************************************************************
--
----------------------------------------------------------------------------
library IEEE ;
use IEEE.STD_LOGIC_1164.ALL ;
use IEEE.STD_LOGIC_ARITH.ALL ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;

entity Stoper is port ( Clk : in std_logic ;
                        Reset : in std_logic ;
                        PushKeyIn : in std_logic ;
                        KeyLed : out std_logic ;
                        RunStatusLed : out std_logic ;
                        Seg_A : out std_logic ;
                        Seg_B : out std_logic ;
                        Seg_C : out std_logic ;
                        Seg_D : out std_logic ;
                        Seg_E : out std_logic ;
                        Seg_F : out std_logic ;
                        Seg_G : out std_logic ;
                        Seg_K : out std_logic ;
                        DSel0 : out std_logic ;
                        DSel1 : out std_logic ;
                        DSel2 : out std_logic ;
                        DSel3 : out std_logic ) ;
end Stoper ;

architecture Behavioral of Stoper is

  constant FDivCntSizeParam : integer := 8 ;
  constant DisplayNegParam : std_logic := '0' ;
  constant DisplayWAParam : std_logic := '0' ;

  component FDivider is generic ( constant CntSize : integer := 8 ) ;
                     port ( Clk : in std_logic ;
                            Reset : in std_logic ;
                            FDiv : out std_logic ) ;
  end component ;

  component PushButton port ( KClk : in std_logic ;
                              Reset : in std_logic ;
                              KeyIn : in std_logic ;
                              Status : out std_logic ) ;
  end component ;

  component BCD4DigCnt port ( Clk : in std_logic ;
                              Reset : in std_logic ;
                              Enable : in std_logic ;
                              Cnt1 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt10 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt100 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt1000 : out std_logic_vector ( 3 downto 0 ) ) ;
  end component ;

  component Mux4DigDisp port ( DClk : in std_logic ;
                               Reset : in std_logic ;
                               Neg : in std_logic := '0' ;
                               WA : in std_logic := '0' ;
                               Tetr0 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr1 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr2 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr3 : in std_logic_vector ( 3 downto 0 ) ;
                               Dot : in std_logic_vector ( 3 downto 0 ) ;
                               Seg_A : out std_logic ;
                               Seg_B : out std_logic ;
                               Seg_C : out std_logic ;
                               Seg_D : out std_logic ;
                               Seg_E : out std_logic ;
                               Seg_F : out std_logic ;
                               Seg_G : out std_logic ;
                               Seg_K : out std_logic ;
                               DSel0 : out std_logic ;
                               DSel1 : out std_logic ;
                               DSel2 : out std_logic ;
                               DSel3 : out std_logic ) ;
  end component ;

  type AutomatStateType is ( ResetState ,
                             IdleState ,
                             StartMeasState ,
                             WaitOnKeyStatusLow ,
                             WaitOnKeyStatusHigh ,
                             StopMeasState ) ;

  signal AutomatState : AutomatStateType ;
  signal KClk : std_logic ;
  signal DClk : std_logic ;
  signal FClk : std_logic ;
  signal FDiv : std_logic_vector ( 3 downto 0 ) ;
  signal TimeDiv : std_logic_vector ( 13 downto 0 ) ;
  signal F100Hz : std_logic ;
  signal KeyStatus : std_logic ;
  signal Clear : std_logic ;
  signal ClrCnt : std_logic ;
  signal Enable : std_logic ;
  signal Cnt1 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt10 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt100 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt1000 : std_logic_vector ( 3 downto 0 ) ;

begin
  FDivInstance : FDivider generic map ( CntSize => FDivCntSizeParam )
                          port map ( Clk => Clk ,
                                     Reset => Reset ,
                                     FDiv => FClk ) ;

  ClkGenInstance : process ( FClk , Reset )
  begin
    if Reset = '0' then
      FDiv <= ( others => '0' ) ;
    else
      if FClk'event and FClk = '0' then
        FDiv <= FDiv + 1 ;
      end if ;
    end if ;
  end process ;

  DClk <= FDiv ( 3 ) ;
  KClk <= FDiv ( 2 ) ;

  AutomatInstance : process ( Clk , Reset )
  begin
    if Reset = '0' then
      AutomatState <= ResetState ;
    else
      if Clk'event and Clk = '1' then
        case AutomatState is
          when ResetState =>
            Clear <= '0' ;
            Enable <= '0' ;
            AutomatState <= IdleState ;
          when IdleState =>
            Clear <= '1' ;
            Enable <= '0' ;
            if KeyStatus = '0' then
              AutomatState <= IdleState ;
            else
              AutomatState <= StartMeasState ;
            end if ;
          when StartMeasState =>
            Clear <= '0' ;
            Enable <= '1' ;
            AutomatState <= WaitOnKeyStatusLow ;
          when WaitOnKeyStatusLow =>
            Clear <= '1' ;
            Enable <= '1' ;
            if KeyStatus = '1' then
              AutomatState <= WaitOnKeyStatusLow ;
            else
              AutomatState <= WaitOnKeyStatusHigh  ;
            end if ;
          when WaitOnKeyStatusHigh =>
            Clear <= '1' ;
            Enable <= '1' ;
            if KeyStatus = '0' then
              AutomatState <= WaitOnKeyStatusHigh ;
            else
              AutomatState <= StopMeasState  ;
            end if ;
          when StopMeasState =>
            Clear <= '1' ;
            Enable <= '0' ;
            if KeyStatus = '1' then
              AutomatState <= StopMeasState  ;
            else
              AutomatState <= IdleState ;
            end if ;
        end case ;
      end if ;
    end if ;
  end process ;

  ClrCnt <= '0' when ( Clear = '0' or Reset = '0' ) else '1' ;

  TimeDivInstance : process ( Clk , ClrCnt , Enable )
  begin
    if ClrCnt = '0' then
      TimeDiv <= ( others => '0' ) ;
    else
      if Enable = '1' then
        if Clk'event and Clk = '1' then
          if TimeDiv = "10011100001111" then
            TimeDiv <= ( others => '0' ) ;
          else
            TimeDiv <= TimeDiv + 1 ;
          end if ;
        end if ;
      end if ;
    end if ;
  end process ;

  F100Hz <= TimeDiv ( 13 ) ;

  Key0Instance : PushButton port map ( KClk => KClk ,
                                       Reset => Reset ,
                                       KeyIn => PushKeyIn ,
                                       Status => KeyStatus ) ;

  KeyLed <= not KeyStatus ;
  RunStatusLed <= not Enable ;

  BCDCntInstance : BCD4DigCnt port map ( Clk => F100Hz ,
                                         Reset => ClrCnt ,
                                         Enable => Enable ,
                                         Cnt1 => Cnt1 ,
                                         Cnt10 => Cnt10 ,
                                         Cnt100 => Cnt100 ,
                                         Cnt1000 => Cnt1000 ) ;

  MuxDisplInstance : Mux4DigDisp port map ( DClk => DClk ,
                                            Reset => Reset ,
                                            Neg => DisplayNegParam ,
                                            WA => DisplayWAParam ,
                                            Tetr0 => Cnt1 ,
                                            Tetr1 => Cnt10 ,
                                            Tetr2 => Cnt100 ,
                                            Tetr3 => Cnt1000 ,
                                            Dot => "1011" ,
                                            Seg_A => Seg_A ,
                                            Seg_B => Seg_B ,
                                            Seg_C => Seg_C ,
                                            Seg_D => Seg_D ,
                                            Seg_E => Seg_E ,
                                            Seg_F => Seg_F ,
                                            Seg_G => Seg_G ,
                                            Seg_K => Seg_K ,
                                            DSel0 => DSel0 ,
                                            DSel1 => DSel1 ,
                                            DSel2 => DSel2 ,
                                            DSel3 => DSel3 ) ;
end Behavioral ;
